/***************************************************************************
                          kscrollinglabel.cpp - scrolling label
                             -------------------
    begin                : 2002-08-28
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kscrollinglabel.h"
#include <qpainter.h>
#include <iostream>

using namespace std;

void
KScrollingLabel::initialise ()
{
  speed = 2;
  delay = 125;
  center_delay = 8;
  center_waited = 0;
  offsetTimerID = -1;
  checkScrolling ();  
  setMinimumWidth (1);
}

void
KScrollingLabel::checkScrolling ()
{
  bool needTimer = (width () < fontMetrics ().width (text ()));
  bool haveTimer = (offsetTimerID != -1);
  if (haveTimer && !needTimer)
    {
      killTimer (offsetTimerID);
      offsetTimerID = -1;
      offset = 0;
      setAlignment (AlignCenter);
    }
  if (needTimer && !haveTimer)
    {
      offsetTimerID = startTimer (delay);
      setAlignment (AlignLeft | AlignVCenter);
    } 
}

KScrollingLabel::KScrollingLabel (QWidget *parent, const char *name, WFlags f)
 : QLabel (parent, name, f)
{
  initialise ();  
}

KScrollingLabel::KScrollingLabel (const QString& text, QWidget *parent, const char *name, WFlags f)
 : QLabel (text, parent, name, f)
{
  initialise ();
}

KScrollingLabel::KScrollingLabel (QWidget *buddy, const QString& text, QWidget *parent, const char *name, WFlags f)
 : QLabel (buddy, text, parent, name, f)
{
  initialise ();
}

void
KScrollingLabel::setText (const QString &text)
{
  QLabel::setText (text);
  checkScrolling ();
}

void
KScrollingLabel::drawContents (QPainter *p)
{
  if (offsetTimerID == -1)
    QLabel::drawContents (p);
  else
    p->drawText (offset, fontMetrics ().ascent(), text ());
}

void
KScrollingLabel::timerEvent (QTimerEvent *tev)
{
  if (tev -> timerId () == offsetTimerID)
    {
      if (offset > -speed && offset < speed)
        {
          offset = 0;
          ++center_waited;
          if (center_waited > center_delay)
            {
              offset -= speed;
              center_waited = 0;
            }
        }
      else
        offset -= speed;
      if (offset < -(fontMetrics ().width (text())))
        offset = width ();
      update ();
    }
}

void
KScrollingLabel::resizeEvent (QResizeEvent *rev)
{
  QLabel::resizeEvent (rev);
  checkScrolling ();
}
