/***************************************************************************
                          kxinecontrol.cpp  -  description
                             -------------------
    begin                : Fri Jun 29 2002
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxinecontrol.h"
#include "kxine.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <qsignalmapper.h>
#include <qtooltip.h>

#include <kiconloader.h>
#include <klocale.h>
#include <kaction.h>

#include <iostream>

using namespace std;

KxineControl::KxineControl(QWidget *parent, const char *name ) : QWidget(parent,name)
{
  QHBoxLayout *ohb = new QHBoxLayout (this, 4, 4);

  playButton = new QToolButton (this);
  playButton -> setIconSet (MainBarIcon ("player_play"));
  playButton -> setAutoRaise (true);
  playButton -> setUsesBigPixmap (true);
  ohb -> addWidget (playButton);

  QVBoxLayout *ivb = new QVBoxLayout (ohb, 4);

  timebar = new KxineSlider (Qt::Horizontal, this);
  timebar -> setTickmarks (QSlider::NoMarks);
  timebar -> setMinValue (0);
  timebar -> setMaxValue (0);
  timebar -> setLineStep (15);
  timebar -> setPageStep (60);
  //  timebar -> setTracking (false);
  ivb -> addWidget (timebar);

  QHBoxLayout *ihb = new QHBoxLayout (ivb, 0);

  QToolButton *stopButton = new QToolButton (this);
  QToolButton *prevButton = new QToolButton (this);
  QToolButton *rewdButton = new QToolButton (this);
  QToolButton *ffwdButton = new QToolButton (this);
  QToolButton *nextButton = new QToolButton (this);
  volbar = new KxineSlider (Qt::Horizontal, this);
  volbar -> setMinValue (0);
  volbar -> setMaxValue (100);
  volbar -> setLineStep (1);
  volbar -> setPageStep (10);
  stopButton -> setIconSet (MainBarIcon ("player_stop"));
  prevButton -> setIconSet (MainBarIcon ("player_start"));
  rewdButton -> setIconSet (MainBarIcon ("player_rew"));
  ffwdButton -> setIconSet (MainBarIcon ("player_fwd"));
  nextButton -> setIconSet (MainBarIcon ("player_end"));
  stopButton -> setAutoRaise (true);
  prevButton -> setAutoRaise (true);
  rewdButton -> setAutoRaise (true);
  ffwdButton -> setAutoRaise (true);
  nextButton -> setAutoRaise (true);
  ihb -> addWidget (stopButton);
  ihb -> addSpacing (4);
  ihb -> addWidget (prevButton);
  ihb -> addWidget (rewdButton);
  ihb -> addWidget (ffwdButton);
  ihb -> addWidget (nextButton);
  ihb -> addSpacing (4);
  ihb -> addStretch (1);
  ihb -> addWidget (volbar);

  rewdButton -> setEnabled (false);

  connect (playButton, SIGNAL (clicked()), SIGNAL (playClicked()));
  connect (stopButton, SIGNAL (clicked()), SIGNAL (stopClicked()));
  connect (prevButton, SIGNAL (clicked()), SIGNAL (prevClicked()));
  connect (rewdButton, SIGNAL (clicked()), SIGNAL (rewdClicked()));
  connect (ffwdButton, SIGNAL (clicked()), SIGNAL (ffwdClicked()));
  connect (nextButton, SIGNAL (clicked()), SIGNAL (nextClicked()));
  connect (volbar, SIGNAL (xineValueChanged(int)), SLOT (volbarChanged(int)));
  connect (timebar, SIGNAL (xineValueChanged(int)), SLOT (timebarChanged(int)));

}

void
KxineControl::setMode (bool playing, bool seeking)
{
  if (playing ^ seeking)
    playButton -> setIconSet (MainBarIcon ("player_pause"));
  else
    playButton -> setIconSet (MainBarIcon ("player_play"));

}

void
KxineControl::setFile (QString, int length)
{
  timebar -> setMaxValue (length);
}

void
KxineControl::setPosition (int pos)
{
  timebar -> xineSetValue (pos);
}

void
KxineControl::setVolume (int vol)
{
  volbar -> xineSetValue (vol);
}


void
KxineControl::timebarChanged (int value)
{
  cerr << "seek to " << value << endl;
  emit seekClicked (timebar->value());
}

void
KxineControl::volbarChanged (int vol)
{
  emit volumeChanged (vol);
}

