/***************************************************************************
                          kxinefiledialog.cpp  -  file dialog
                             -------------------
    begin                : 2002-08-10
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxinefiledialog.h"

#include <kfile.h>

#include "kxine.h"

KxineFileDialog* KxineFileDialog::current = NULL;

KxineFileDialog::KxineFileDialog (QWidget *parent, const char *name)
  : KFileDialog (":open", QString::null, parent, name, false)
{
  setOperationMode (KFileDialog::Opening);
  setMode (static_cast<KFile::Mode>(KFile::Files
                                  | KFile::ExistingOnly
				  | KFile::LocalOnly
				  ));
  current = this;  
}

KxineFileDialog::~KxineFileDialog ()
{
  current = NULL;
}

KxineFileDialog*
KxineFileDialog::getInstance (QWidget *parent, const char *id)
{
  if (current != NULL)
    delete current;
  return new KxineFileDialog (parent, id);
}

void
KxineFileDialog::slotOk ()
{
  KFileDialog::slotOk ();
  QStringList files = selectedFiles ();
  if (files.count () > 0)
    {
      emit firstFile (files.front ());
      if (files.count () > 1)
        {
          QStringList::Iterator it = files.begin ();
	  it++;
	  while (it != files.end ())
	    {
              cerr << "emitting otherFile " << *it << endl;
	      emit otherFile (*(it++));
	    }
	}
    }
  delayedDestruct ();
}

void
KxineFileDialog::accept ()
{
  KFileDialog::accept ();
}

void
KxineFileDialog::slotCancel ()
{
  KFileDialog::slotCancel ();
  delayedDestruct ();
}

void
KxineFileDialog::closeEvent (QCloseEvent *cev)
{
  cev -> accept ();
  delayedDestruct ();
}
