/***************************************************************************
                          kxinelistbox.cpp
                             -------------------
    begin                : 2002-09-07
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxinelistbox.h"
#include "kxinelistitem.h"

KxineListBox::KxineListBox (QWidget *parent, const char *name, WFlags f)
  : KListBox (parent, name, f)
{

  playingItem = NULL;
  playingNext = NULL;
  connect (this, SIGNAL (executed(QListBoxItem*)), SLOT (slotExecuted(QListBoxItem*)));

}

void
KxineListBox::removeItem (int index)
{
  KxineListItem *ritem = reinterpret_cast <KxineListItem *>(item (index));
  bool restore_next = playingNext == ritem && playingItem && playingItem -> next () == ritem;
  if (playingItem == ritem)
    playingItem = NULL;
  if (playingNext == ritem)
    playingNext = NULL;
  KListBox::removeItem (index);
  if (restore_next && playingItem)
    playingNext = reinterpret_cast <KxineListItem *>(playingItem -> next ());  
}

void
KxineListBox::setPlaying (KxineListItem *p_item)
{
  setPlayingItem (p_item);
  if (playingItem == NULL)
    {
      if (count () == 0)
        setPlayingNext (NULL);
      else
        setPlayingNext (reinterpret_cast <KxineListItem*>(item (0)));
    }
  else
    {
      KxineListItem *next_item =
        reinterpret_cast <KxineListItem*>(p_item -> next());
      setPlayingNext (next_item);
    }
  triggerUpdate (false);
}

void
KxineListBox::setPlayingItem (KxineListItem *item)
{
  playingItem = item;
}

void
KxineListBox::setPlayingNext (KxineListItem *item)
{
  playingNext = item;
}

void
KxineListBox::slotExecuted (QListBoxItem *q_item)
{
  KxineListItem *item = reinterpret_cast<KxineListItem*>(q_item);
  setPlaying (item);
  emit playMRL (item -> getMRL ());
}

void
KxineListBox::keyPressEvent (QKeyEvent *kev)
{
  KListBox::keyPressEvent (kev);
  kev -> ignore ();  /* pretend we ignore it in case it is a shortcut */
}

void
KxineListBox::mouseMoveEvent (QMouseEvent *mev)
{
  if (mev -> state () & LeftButton)
    {
      int current_id = currentItem ();
      int mouseon_id = index (itemAt (mev->pos ()));
      if (current_id >= 0 && mouseon_id >= 0 && current_id != mouseon_id)
        {
          KxineListItem * current = reinterpret_cast <KxineListItem*>(item (current_id));
          KxineListItem * mouseon = reinterpret_cast <KxineListItem*>(item (mouseon_id));
          bool restore_next = getPlaying () && getPlayingNext () == getPlaying () -> next ();
          takeItem (current);
          insertItem (current, (mouseon_id > current_id) ? mouseon : mouseon -> prev ());
          setCurrentItem (current);
          if (restore_next)
            setPlayingNext (reinterpret_cast <KxineListItem *>(getPlaying () -> next ()));
          triggerUpdate (false);
        }
    }
  else
    {
      KListBox::mouseMoveEvent (mev);
    }

}
