/***************************************************************************
                          kxinemenu.cpp  -  description
                             -------------------
    begin                : Fri Jun 28 2002
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxinemenu.h"
#include "kxinefiledialog.h"

#include "kxine.h"

#include <khelpmenu.h>
#include <klocale.h>
#include <kaccel.h>
#include <kfiledialog.h>
#include <klineeditdlg.h>

#include <qlayout.h>

#include <iostream>

using namespace std;

KxineMenu::KxineMenu(QWidget *parent, const char *name ) : QWidget(parent,name)
{
  QVBoxLayout *vbl = new QVBoxLayout (this);
  menubar = new KMenuBar (this);
  videoContext = new KPopupMenu (this);

  // File menu
  KPopupMenu *fileMenu = new KPopupMenu (this);

  actionCollection = kxine->actionCollection ();

  KAction *openFile = new KAction (i18n ("&Open..."), "fileopen",
                                   KStdAccel::shortcut (KStdAccel::Open),
                                   this, SLOT (fileOpen ()),
                                   actionCollection, "openfile");

  KAction *enquFile = new KAction (i18n ("&Enqueue..."), "fileopen",
                                   0,
                                   this, SLOT (fileEnqueue ()),
                                   actionCollection, "enqueuefile");

  autoplayInputPlugins = new KPopupMenu (this);

  KAction *openMRL = new KAction (i18n ("Open &MRL..."), "fileopen",
                                  0,
                                  this, SLOT (fileOpenMRL ()),
                                  actionCollection, "openmrl");

  KAction *exitKxine = new KAction (i18n ("E&xit"), "exit",
                                    KStdAccel::shortcut (KStdAccel::Quit),
                                    this, SIGNAL (appClose ()),
                                    actionCollection, "exit");

  openFile->plug (fileMenu);
  enquFile->plug (fileMenu);
  fileMenu->insertItem (i18n("O&pen Other"), autoplayInputPlugins);
  openMRL ->plug (fileMenu);
  fileMenu->insertSeparator ();
  exitKxine->plug (fileMenu);

  // View Menu
  KPopupMenu *viewMenu = new KPopupMenu (this);

  KToggleAction *viewMenuFullscreen = new KToggleAction 
                            (i18n ("&Fullscreen"), "window_fullscreen", 0,
                             actionCollection, "fullscreen");


  connect (viewMenuFullscreen, SIGNAL (toggled(bool)), SIGNAL (toggleFullScreen(bool)));

  viewMenuFullscreen -> plug (viewMenu);
  viewMenuFullscreen -> plug (videoContext);

  KPopupMenu * viewZoomMenu = new KPopupMenu (this);
  QSignalMapper *viewZoomMapper = new QSignalMapper (this);

  KAction *zoomHalfSize = new KAction (i18n ("&Half Size"), "", 0,
                                       viewZoomMapper, SLOT (map()),
                                       actionCollection, "zoomhalf");

  KAction *zoomNormalSize = new KAction (i18n ("&Normal Size"), "", 0,
                                       viewZoomMapper, SLOT (map()),
                                       actionCollection, "zoomnormal");

  KAction *zoomDoubleSize = new KAction (i18n ("&Double Size"), "", 0,
                                       viewZoomMapper, SLOT (map()),
                                       actionCollection, "zoomdouble");

  viewZoomMapper -> setMapping (zoomHalfSize, HALF_SIZE);
  viewZoomMapper -> setMapping (zoomNormalSize, NORMAL_SIZE);
  viewZoomMapper -> setMapping (zoomDoubleSize, DOUBLE_SIZE);

  zoomHalfSize -> plug (viewZoomMenu);
  zoomNormalSize -> plug (viewZoomMenu);
  zoomDoubleSize -> plug (viewZoomMenu);

  connect (viewZoomMapper, SIGNAL (mapped(int)), SLOT (viewZoom(int)));

  viewMenu -> insertItem (i18n ("&Zoom"), viewZoomMenu);

  // Options Menu
  KPopupMenu *optionsMenu = new KPopupMenu (this);

  KToggleAction *optionsMenuDeinterlace = new KToggleAction
                             (i18n ("De&interlace"), 0, 0,
                              actionCollection, "deinterlace");
                             
  connect (optionsMenuDeinterlace, SIGNAL (toggled(bool)), SIGNAL (toggleDeinterlace(bool)));

  optionsMenuDeinterlace -> plug (optionsMenu);
  optionsMenuDeinterlace -> plug (videoContext);

  KAction *videoControl = new KAction (i18n ("&Video Controls..."), "", 0,
                                       this, SIGNAL (openVideoControlWindow()),
                                       actionCollection, "videocontrol");

  videoControl -> plug (optionsMenu);

  // Help Menu
  KHelpMenu *helpMenu = new KHelpMenu (this);
  connect (helpMenu, SIGNAL (showAboutApplication ()),
	   SIGNAL (aboutApplication ()));

  // The menu bar
  menubar->insertItem (i18n ("&File"), fileMenu);
  menubar->insertItem (i18n ("&View"), viewMenu);
  menubar->insertItem (i18n ("&Options"), optionsMenu);
  menubar->insertItem (i18n ("&Help"), helpMenu->menu ());

  videoContext -> insertItem (i18n ("&Zoom"), viewZoomMenu);

  vbl -> addWidget (menubar);
  menubar -> setSizePolicy ( QSizePolicy (QSizePolicy::MinimumExpanding,
                                          QSizePolicy::Fixed) );
  setSizePolicy ( QSizePolicy (QSizePolicy::MinimumExpanding,
                               QSizePolicy::Fixed) );
}

QSize
KxineMenu::minimumSize ()
{
  return menubar -> sizeHint ();
}

QSize
KxineMenu::sizeHint ()
{
  return menubar -> sizeHint ();
}

void
KxineMenu::setFullscreenCheck (bool c)
{
  KAction *act = actionCollection -> action("fullscreen");
  KToggleAction *tact = dynamic_cast <KToggleAction *> (act);
  tact -> setChecked (c);
}

void
KxineMenu::setDeinterlaceCheck (bool d)
{
  KAction *act = actionCollection -> action("deinterlace");
  KToggleAction *tact = dynamic_cast <KToggleAction *> (act);
  tact -> setChecked (d);
}


void
KxineMenu::fileOpen ()
{
  cerr << "creating dialog" << endl;
  KxineFileDialog *dialog = KxineFileDialog::getInstance (this, "fileopendlg");

  cerr << "connecting signals" << endl;
  connect (dialog, SIGNAL (firstFile(QString)), SIGNAL (loadMRL(QString)));
  connect (dialog, SIGNAL (otherFile(QString)), SIGNAL (enqueueMRL(QString)));

  cerr << "showing dialog" << endl;
  dialog -> show ();

  cerr << "done" << endl;
}

void
KxineMenu::fileEnqueue ()
{
  KxineFileDialog* dialog = KxineFileDialog::getInstance (this, "fileenqdlg");

  connect (dialog, SIGNAL (firstFile(QString)), SIGNAL (enqueueMRL(QString)));
  connect (dialog, SIGNAL (otherFile(QString)), SIGNAL (enqueueMRL(QString)));
  
  dialog -> show ();
}

void
KxineMenu::fileOpenMRL ()
{
  bool ok;
  QString fileName =
    KLineEditDlg::getText ("Enter MRL:", "", &ok, this);
  if (ok && fileName != QString::null)
    {
      emit loadMRL (fileName);
    }
}

void
KxineMenu::viewZoom (int level)
{
  switch (level)
    {
    case HALF_SIZE:
      emit zoomChanged (0.5);
      break;
    case DOUBLE_SIZE:
      emit zoomChanged (2.0);
      break;
    case NORMAL_SIZE:
    default:
      emit zoomChanged (1.0);
      break;
    }
  cerr << "emitting adjustSize" << endl;
  emit frameAdjustSize ();
}

void
KxineMenu::addAutoplayInputPlugin (const QString &plugin_name, QSignalMapper *mapper,
                                   const char *slot, const char *id)
{
  KAction *plugin = new KAction (plugin_name, 0, 0,
				 mapper, slot,
				 actionCollection);
  mapper -> setMapping (plugin, id);
  plugin -> plug (autoplayInputPlugins);
}
