/***************************************************************************
                          kxinenow.cpp
                             -------------------
    begin                : 2002-08-28
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxinenow.h"
#include "kxinefiledialog.h"
#include "kxine.h"
#include <qlayout.h>
#include <qsignalmapper.h>
#include <qframe.h>
#include <qstringlist.h>
#include <qtooltip.h>
#include <qpopupmenu.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kaction.h>
#include "kxinelistitem.h"

KxineNow::KxineNow (QWidget *parent, const char *name)
 : QWidget (parent, name)
{
  QVBoxLayout *vbl = new QVBoxLayout (this, 2, 2);

  QFrame *box = new QFrame (this);
  box -> setFrameStyle (QFrame::Panel + QFrame::Sunken);
  box -> setLineWidth (1);
  QVBoxLayout *boxvbl = new QVBoxLayout (box, 2, 1);

  title = new KScrollingLabel (i18n ("No Stream Selected"), box, "title");
  boxvbl -> addWidget (title);  
  position = new QLabel ("", box, "pos");
  position -> setAlignment (AlignCenter);
  boxvbl -> addWidget (position);

  vbl -> addWidget (box);
  vbl -> addSpacing (4);
  vbl -> addWidget (queue_label = new QLabel (i18n ("Coming Next:"), this));

  queue = new KxineListBox (this, "queue");
  connect (queue, SIGNAL (playMRL (QString)), SIGNAL (playMRL (QString)));
  vbl -> addWidget (queue, 1);

  QHBoxLayout *hbl = new QHBoxLayout (vbl, 2, 0);

  hbl -> addStretch (1);
  addButton = new QToolButton (this);
  addButton -> setIconSet (MainBarIcon ("edit_add"));
  addButton -> setAutoRaise (true);
  QToolTip::add (addButton, i18n ("Enqueue file"));
  connect (addButton, SIGNAL (clicked()), SLOT (addPressed()));
  hbl -> addWidget (addButton);
  removeButton = new QToolButton (this);
  removeButton -> setIconSet (MainBarIcon ("edit_remove"));
  removeButton -> setAutoRaise (true);
  QToolTip::add (removeButton, i18n ("Dequeue file"));
  connect (removeButton, SIGNAL (clicked()), SLOT (removePressed()));
  hbl -> addWidget (removeButton);

  menuButton = new QToolButton (this);
  menuButton -> setIconSet (MainBarIcon ("contents"));
  menuButton -> setAutoRaise (true);
  menuButton -> setTextLabel (i18n ("DVD Menu"), true);
  hbl -> addWidget (menuButton);

  QSignalMapper *menuMapper = new QSignalMapper (this);
  QPopupMenu *menuPopup = new QPopupMenu (this);
  menuButton -> setPopup (menuPopup);
  connect (menuButton, SIGNAL (clicked ()), menuMapper, SLOT (map()));

  KActionCollection *actions = kxine->actionCollection ();
  KAction *menuRoot = new KAction (i18n("&Root Menu"), 0, 0,
                                   menuMapper, SLOT (map ()),
                                   actions, "dvd_rootmenu");
  KAction *menuDisc = new KAction (i18n("&Disc Menu"), 0, 0,
                                   menuMapper, SLOT (map ()),
                                   actions, "dvd_discmenu");
  KAction *menuTitle = new KAction (i18n("&Title Menu"), 0, 0,
                                    menuMapper, SLOT (map ()),
                                    actions, "dvd_titlemenu");

  menuRoot -> plug (menuPopup);
  menuDisc -> plug (menuPopup);
  menuTitle -> plug (menuPopup);

  menuMapper -> setMapping (menuButton, 1);
  menuMapper -> setMapping (menuRoot, 1);
  menuMapper -> setMapping (menuDisc, 2);
  menuMapper -> setMapping (menuTitle, 3);

  connect (menuMapper, SIGNAL (mapped(int)), SIGNAL (menuClicked(int)));

  menuButton -> hide ();

  hbl -> addStretch (1);

  pos_len = QString::null;
  setPosition (0);

}

void
KxineNow::setPosition (int pos)
{
  pos_cur.sprintf("%d:%02d:%02d", pos/3600, (pos/60) % 60, pos % 60);
  if (pos_len != QString::null)
    position -> setText(QString(i18n("%1 of %2")).arg(pos_cur).arg(pos_len));
  else
    position -> setText(pos_cur);
}

void
KxineNow::setLength (int len)
{
  if (len == 0)
    {
      pos_len = QString::null;
      position -> setText (pos_cur);
    }
  else
    {
      pos_len.sprintf("%d:%02d:%02d", len/3600, (len/60) % 60, len % 60);
      position -> setText(QString(i18n("%1 of %2")).arg(pos_cur).arg(pos_len));
    }
}

void
KxineNow::setChapterMode ()
{
  if (!chapterMode)
    {
      chapterMode = true;
      queue_label -> setText (i18n("Chapters:"));
      queue -> clear ();
      menuButton -> show ();
      addButton -> hide ();
      removeButton -> hide ();
    }
}

void
KxineNow::setQueueMode ()
{
  if (chapterMode)
    {
      chapterMode = false;
      queue_label -> setText (i18n("Coming Next:"));
      queue->clear ();
      addButton -> show ();
      removeButton -> show ();
      menuButton -> hide ();
    }
}

void
KxineNow::enqueueMRL (QString mrl)
{
  setQueueMode ();
  new KxineListItem (queue, mrl); 
}

void
KxineNow::enqueueMRLs (const QStringList &mrls)
{
  setQueueMode ();
  for (QStringList::const_iterator iter = mrls.begin (); iter != mrls.end (); iter++)
    new KxineListItem (queue, *iter);
}

void
KxineNow::clearQueue ()
{
  setQueueMode ();
  queue -> clear ();
}

void
KxineNow::setChapters (const QStringList& chaps)
{
  setChapterMode ();
  queue -> clear ();
  for (QStringList::const_iterator iter = chaps.begin (); iter != chaps.end (); iter++)
    new KxineListItem (queue, *iter);
}

void
KxineNow::clearChapters ()
{
  setChapterMode ();
  queue -> clear ();
}

QString
KxineNow::peekNextMRL ()
{
  if (chapterMode)
    {
      KxineListItem *next = queue -> getPlayingNext ();
      if (next == NULL)
        return QString::null;
      else 
        return next -> getMRL ();
    }
  else
    {
      if (queue -> count () == 0)
        return QString::null;
      KxineListItem *next = reinterpret_cast <KxineListItem *>(queue -> item (0));
      return next -> getMRL ();
    }
}

QString
KxineNow::getNextMRL ()
{
  if (chapterMode)
    {
      KxineListItem *next = queue -> getPlayingNext ();
      if (next == NULL)
        {
          queue -> setPlaying (NULL);
          return QString::null;
        }
    
      queue -> setPlaying (next);
      return next -> getMRL ();
    }
  else
    {
      if (queue -> count () == 0)
        return QString::null;
      KxineListItem *next = reinterpret_cast <KxineListItem *>(queue -> item (0));
      QString next_mrl = next -> getMRL ();
      queue -> removeItem (0);
      return next_mrl;
    }
}

void
KxineNow::addPressed ()
{

   KxineFileDialog *dialog = KxineFileDialog::getInstance (this, "fileenq");
   connect (dialog, SIGNAL (firstFile(QString)), SLOT (enqueueMRL(QString)));
   connect (dialog, SIGNAL (otherFile(QString)), SLOT (enqueueMRL(QString)));
   dialog -> show ();
}

void
KxineNow::removePressed ()
{
  if (queue -> currentItem () != -1)
    {
      queue -> removeItem (queue -> currentItem ());
    }
}
