/***************************************************************************
                          kxineplaylist.cpp
                             -------------------
    begin                : 2002-09-07
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxineplaylist.h"

#include "kxinefiledialog.h"
#include "kxinelistitem.h"

#include <kiconloader.h>
#include <klocale.h>

#include <qlayout.h>
#include <qtooltip.h>

KxinePlayList::KxinePlayList (QWidget *parent, const char *name, WFlags f)
  : QWidget (parent, name, f)
{

  QVBoxLayout *vbl = new QVBoxLayout (this, 2, 2);

  list = new KxineListBox (this, "playlistbox");
  vbl -> addWidget (list, 1);

  QHBoxLayout *hbl = new QHBoxLayout (vbl, 2, 0);

  hbl -> addStretch (1);
  addButton = new QToolButton (this);
  addButton -> setIconSet (MainBarIcon ("edit_add"));
  addButton -> setAutoRaise (true);
  QToolTip::add (addButton, i18n ("Add to play list"));
  connect (addButton, SIGNAL (clicked()), SLOT (addPressed()));
  hbl -> addWidget (addButton);
  removeButton = new QToolButton (this);
  removeButton -> setIconSet (MainBarIcon ("edit_remove"));
  removeButton -> setAutoRaise (true);
  QToolTip::add (removeButton, i18n ("Remove from play list"));
  connect (removeButton, SIGNAL (clicked()), SLOT (removePressed()));
  hbl -> addWidget (removeButton);
  repeatButton = new QToolButton (this);
  repeatButton -> setIconSet (MainBarIcon ("redo"));
  repeatButton -> setAutoRaise (true);
  repeatButton -> setToggleButton (true);
  QToolTip::add (repeatButton, i18n ("Repeat play list"));
  hbl -> addWidget (repeatButton);
  hbl -> addStretch (1);

  connect (list, SIGNAL (playMRL(QString)), SIGNAL (playMRL(QString)));
}

QString
KxinePlayList::peekNextMRL ()
{
  KxineListItem *next = list -> getPlayingNext ();
  if (next == NULL)
    {
      if (repeatButton -> isOn ())
        return reinterpret_cast <KxineListItem *>(list -> item (0)) -> getMRL ();
      else
        return QString::null;
    }
  else 
    {
      return next -> getMRL ();
    }
}

QString
KxinePlayList::getNextMRL ()
{
  KxineListItem *next = list -> getPlayingNext ();
  if (next == NULL)
    {
      if (repeatButton -> isOn ())
        {
          next = reinterpret_cast <KxineListItem *>(list -> item (0));
        }
      else
        {
          list -> setPlaying (NULL);
          return QString::null;
        }
    }

  list -> setPlaying (next);
  return next -> getMRL ();
}

void
KxinePlayList::addPressed ()
{
   KxineFileDialog *dialog = KxineFileDialog::getInstance (this, "playlistfileadd");
   connect (dialog, SIGNAL (firstFile(QString)), SLOT (playlistAppend(QString)));
   connect (dialog, SIGNAL (otherFile(QString)), SLOT (playlistAppend(QString)));
   dialog -> show ();
}

void
KxinePlayList::removePressed ()
{
  if (list -> currentItem () != -1)
    {
      list -> removeItem (list -> currentItem ());
    }
}

void
KxinePlayList::playlistAppend (QString mrl)
{
  new KxineListItem (list, mrl);
}
