/***************************************************************************
                          kxinevideo.cpp  -  description
                             -------------------
    begin                : Sun May 19 09:37:48 BST 2002
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxinevideo.h"

#include <kxine.h>
#include <klocale.h>

#include <qsignalmapper.h>
#include <qdragobject.h>
#include <qcursor.h>

#include <X11/extensions/XShm.h>

#ifdef HAVE_XTESTEXTENSION
#include <X11/extensions/XTest.h>
#endif

#include <xine/video_out_x11.h>

#include <iostream>
#include <cstdlib>
#include <cmath>
#include <cstring>

using namespace std;

// some nasty hacky icky XShm stuff
#ifndef XShmGetEventBase
extern "C" {
  extern int XShmGetEventBase(Display *);
}
#endif

KxineVideo::KxineVideo (KxineSideBar *s, KxineMenu *m, KxineControl *c,
                        QWidget * parent, const char *name)
      : QWidget (parent, name)
{
  timerId = -1;
  videoSize = QSize (320, 240);
  sizeMultiple = 1.0;
  menu = m;
  control = c;
  sidebar = s;

  setPaletteBackgroundColor (QColor (0, 0, 0));
  setUpdatesEnabled (false);
  setAcceptDrops (true);
  setMouseTracking (true);

  connect (menu, SIGNAL (loadMRL(QString)), kxine, SLOT (playFile(QString)));
  connect (menu, SIGNAL (zoomChanged(double)), SLOT (xineSetZoom(double)));

  connect (control, SIGNAL (playClicked()), SLOT (xinePlayPause()));
  connect (control, SIGNAL (stopClicked()), SLOT (xineStop()));
  connect (control, SIGNAL (prevClicked()), SLOT (xinePrevious()));
  connect (control, SIGNAL (nextClicked()), SLOT (xineNext()));
  connect (control, SIGNAL (rewdClicked()), SLOT (xineRewind()));
  connect (control, SIGNAL (ffwdClicked()), SLOT (xineFastForward()));
  connect (control, SIGNAL (seekClicked(int)), SLOT (xineSeek(int)));
  connect (control, SIGNAL (volumeChanged(int)), SLOT (xineSetVolume(int)));

  connect (sidebar, SIGNAL (playMRL(QString)), kxine, SLOT (playFile(QString)));
  connect (sidebar, SIGNAL (menuClicked(int)), SLOT (xineMenu(int)));

  connect (menu, SIGNAL (enqueueMRL(QString)), kxine, SLOT (enqueueFile(QString)));

  connect (this, SIGNAL (modeChange(bool,bool)), control, SLOT (setMode(bool,bool)));
  connect (this, SIGNAL (fileChange(QString,int)), control, SLOT (setFile(QString, int)));
  connect (this, SIGNAL (positionChange(int)), control, SLOT (setPosition(int)));
  connect (this, SIGNAL (positionChange(int)), sidebar, SLOT (setPosition(int)));

}

void
KxineVideo::polish ()
{
  QWidget::polish ();

  if (!XInitThreads ())
    {
      cerr << "Kxine: could not initialise X threads." << endl
           << "Kxine: ensure you have a threadsafe Xlib installed." << endl;
      exit (4);
    }

  xineDisplay = XOpenDisplay (NULL);
  XLockDisplay (xineDisplay);

  QString homedir = std::getenv("HOME");
  if (homedir == QString::null)
    homedir = "/tmp";
  QString xinerc = QString("%1/%2").arg(homedir).arg(".xine/config");
  xineConf = xine_config_file_init (const_cast <char *> (static_cast <const char*> (
                                     xinerc.local8Bit())));

  // Get the X visual data to pass to xine
  x11_visual_t xvt;
  xvt.display = xineDisplay;
  xvt.screen = DefaultScreen (xineDisplay);
  xvt.display_ratio = 1;

  double res_h = DisplayWidth (xvt.display, xvt.screen) * 1000 /
                 DisplayWidthMM (xvt.display, xvt.screen);
  double res_v = DisplayHeight (xvt.display, xvt.screen) * 1000 /
                 DisplayHeightMM (xvt.display, xvt.screen);
  xvt.display_ratio = res_h / res_v;
  if (fabs (xvt.display_ratio - 1.0) < 0.01)
    xvt.display_ratio = 1.0;
 
  xvt.d = winId ();
  xvt.user_data = this;
  xvt.dest_size_cb = &KxineVideo::destSize_cb;
  xvt.frame_output_cb = &KxineVideo::frameOutput_cb;

  XSelectInput (xineDisplay, winId (), ExposureMask | StructureNotifyMask);

  char *audioPlugin =
      xineConf -> register_string (xineConf,
                                   const_cast<char *>("audio.driver"),
                                   const_cast<char *>("auto"),
                                   const_cast<char *>("audio driver to use"),
                                   NULL, NULL, NULL);

  if (strcmp (audioPlugin, "auto"))
    {
      audioDriver = xine_load_audio_output_plugin (xineConf, audioPlugin);
    }
  else
    {
      char **driver_ids = xine_list_audio_output_plugins ();
      int i = 0;
      while (driver_ids[i])
        {
          audioPlugin = driver_ids[i];
          audioDriver = xine_load_audio_output_plugin (xineConf, audioPlugin);
          if (audioDriver)
            {
              xineConf->update_string (xineConf, const_cast<char *>("audio.driver"), audioPlugin);
              break;
            }
          ++i;
        }
    }
 
  char *videoPlugin =
      xineConf -> register_string (xineConf,
                                   const_cast<char *>("video.driver"),
                                   const_cast<char *>("auto"),
                                   const_cast<char *>("video driver to use"),
                                   NULL, NULL, NULL);

  if (strcmp (videoPlugin, "auto"))
    {
      videoDriver = xine_load_video_output_plugin (xineConf, videoPlugin,
                                                   VISUAL_TYPE_X11, (void *) &xvt);
    }
  else
    {
      char **driver_ids = xine_list_video_output_plugins (VISUAL_TYPE_X11);
      int i = 0;
      while (driver_ids[i])
        {
          videoPlugin = driver_ids[i];
          videoDriver = xine_load_video_output_plugin (xineConf, videoPlugin,
                                                       VISUAL_TYPE_X11, (void *) &xvt);
          if (videoDriver)
            {
              xineConf->update_string (xineConf, const_cast<char *>("video.driver"), videoPlugin);
              break;
            }
          ++i;
        }
    }

  xine = xine_init (videoDriver, audioDriver, xineConf);

  int mixercaps = xine_get_audio_capabilities (xine);
  if(mixercaps & AO_CAP_PCM_VOL)
    mixer = AO_PROP_PCM_VOL;
  else if(mixercaps & AO_CAP_MIXER_VOL)
    mixer = AO_PROP_MIXER_VOL;
  else
    mixer = -1;

  xine_register_event_listener (xine, &KxineVideo::xineEvent_cb, this);

  if (XShmQueryExtension (xineDisplay) == True)
    {
      shmEventId = XShmGetEventBase (xineDisplay) + ShmCompletion;
    } 

  // get the Autoplay Input Plugins
  QSignalMapper *aipmapper = new QSignalMapper (this);

  char **aips = xine_get_autoplay_input_plugin_ids (xine);
  while (*aips)
    {
      QString name = QString ("%1 (%2)").arg(*aips)
                                        .arg(xine_get_input_plugin_description(xine, *aips));
      menu -> addAutoplayInputPlugin (name, aipmapper, SLOT (map()), *aips);
      ++aips;
    }

  connect (aipmapper, SIGNAL(mapped(const QString&)), SLOT (xineOpenAutoplayInputPlugin(const QString&)));

  if (mixer != -1)
    control -> setVolume (xine_get_audio_property (xine, mixer));
  /* FIXME: else disable volume control */

  pthread_create (&xineXEventThread, NULL, &KxineVideo::xineXEventThread_fn, this);

  XUnlockDisplay (xineDisplay);

}

void *
KxineVideo::xineXEventThread_fn (void *v_kxv)
{
  KxineVideo *kxv = static_cast<KxineVideo *>(v_kxv);
  XEvent event;

  while (kxv -> xineDisplay)
    {
      XNextEvent (kxv -> xineDisplay, &event);
      XLockDisplay (kxv -> xineDisplay);
 
      if (event.type == kxv -> shmEventId)
        {
          // shmEventId event id; pass the data onto xine
          kxv->videoDriver->gui_data_exchange (kxv->videoDriver,
                                      GUI_DATA_EX_COMPLETION_EVENT, &event);

        }
      else if (event.type == Expose)
        {
          int count = reinterpret_cast<XExposeEvent*>(&event)->count;
          if (count == 0)
            kxv->videoDriver->gui_data_exchange (kxv->videoDriver,
                                        GUI_DATA_EX_EXPOSE_EVENT, &event);
        }
      XUnlockDisplay (kxv -> xineDisplay);
    }
  pthread_exit (NULL);
  return NULL;
}

void
KxineVideo::destSize_cb (void *v_kxv, int, int, int *dest_w, int *dest_h)
{
  KxineVideo *kxv = static_cast<KxineVideo *>(v_kxv);
  *dest_w = kxv -> width ();
  *dest_h = kxv -> height ();
}

void
KxineVideo::frameOutput_cb (void *v_kxv, int vid_w, int vid_h, int *dest_x,
                            int *dest_y, int *dest_w, int *dest_h, int *win_x,
                            int *win_y)
{
  static int last_width = 0, last_height = 0;
  KxineVideo *kxv = static_cast <KxineVideo *>(v_kxv);
  *dest_x = 0;
  *dest_y = 0;

  // we are in the xine thread, so we must lock Qt
  *dest_w = kxv->width ();
  *dest_h = kxv->height ();
  *win_x = kxv->x ();
  *win_y = kxv->y ();
  if (vid_w != last_width || vid_h != last_height)
    {
      kxv->setVideoSize (QSize (vid_w, vid_h));
      last_width = vid_w;
      last_height = vid_h;
    }
}

void
KxineVideo::xineEvent_cb (void *v_kxv, xine_event_t * ev)
{
  KxineVideo *kxv = (KxineVideo *) v_kxv;
  switch (ev->type)
    {
    case XINE_EVENT_FRAME_CHANGE:
      {

      }
      break;
    case XINE_EVENT_UI_CHANNELS_CHANGED:
      {
        //kxv->playlist->getNow()->getDvdAudio()->
        //   setValue(xine_get_audio_selection (kxv->xine));
        //kxv->playlist->getNow()->getDvdSubtitle()->
        //   setValue(xine_get_spu_channel (kxv->xine));
      }
      break;
    case XINE_EVENT_UI_SET_TITLE:
      {
        kxv->sidebar -> setTitle (
                 static_cast<char *>((reinterpret_cast<xine_ui_event_t *>(
                    ev))->data));
      }
    case XINE_EVENT_NEED_NEXT_MRL:
      {
        xine_next_mrl_event_t *uevent = reinterpret_cast<xine_next_mrl_event_t *>(ev);
        uevent -> handled = 1;
        uevent -> mrl = const_cast<char *> (kxv->xinePeekNextMRL ());
        cerr << "NEXT MRL: " << uevent -> mrl << endl;
        cerr << "--------: " << kxv->xinePeekNextMRL () << endl;
      }
      break;
    case XINE_EVENT_PLAYBACK_FINISHED:
      {
        cerr << "PLAYBACK FINISHED" << endl;
        kxv -> xineNext ();  
      }
      break;
    case XINE_EVENT_MOUSE_MOVE:
    case XINE_EVENT_MOUSE_BUTTON:
    case XINE_EVENT_INPUT_NEXT:
    case XINE_EVENT_INPUT_PREVIOUS:
      break;
    default:
      cerr << "received unhandled xine event with type " << ev->type << endl;
    }
}

QSize
KxineVideo::getVideoSize ()
{
  return QSize (videoSize.width() * sizeMultiple, videoSize.height() * sizeMultiple);
}

int
KxineVideo::xineGetProperty (int prop)
{
  return videoDriver -> get_property (videoDriver, prop);
}

void
KxineVideo::xineGetPropertyMinMax (int prop, int *min_p, int *max_p)
{
  videoDriver -> get_property_min_max (videoDriver, prop, min_p, max_p);
}

bool
KxineVideo::xineCheckCapability (int cap)
{
  return (cap & (videoDriver -> get_capabilities (videoDriver))) != 0;
}

char *
KxineVideo::xinePeekNextMRL ()
{
  return strdup (sidebar -> peekNextMRL ().local8Bit());
}

void
KxineVideo::resetScreenSaver ()
{
  XLockDisplay (xineDisplay);
  XResetScreenSaver (xineDisplay);
  XUnlockDisplay (xineDisplay);
}

void
KxineVideo::xineOpen (QString mrl)
{
  if (xine_get_status (xine) != XINE_STOP)
    {
      xineStop ();
    }
  currentMRL = mrl;
  int slashPos = mrl.findRev('/');
  if (slashPos < 0)
    sidebar -> setTitle (mrl);
  else
    sidebar -> setTitle (mrl.mid(slashPos+1));
  xinePlayPause ();
}

void
KxineVideo::xineClose ()
{
  xineStop ();
  currentMRL = QString::null;
  sidebar -> setTitle (i18n ("No Stream Selected"));
  sidebar -> setLength (0);
  sidebar -> setPosition (0);
}

void
KxineVideo::xinePlayPause ()
{
  if (currentMRL == "")
    {
      xineNext ();
      return;
    }

  if (xine_get_status (xine) != XINE_PLAY)
    {
      xine_play (xine, const_cast<char *> (
                 static_cast<const char *> (currentMRL.local8Bit ())), 0, 0);
      xine_set_speed (xine, SPEED_NORMAL);
      int length = xine_get_stream_length (xine);
      sidebar -> setLength (length);   /* FIXME: should be Signal/Slot */
      emit fileChange (currentMRL, length);
      emit modeChange (true, false);
      if (timerId == -1)
        timerId = startTimer (500);
    }
  else
    {
      // file is playing or was paused
      if (xine_get_speed (xine) == SPEED_NORMAL)
        {
          xine_set_speed (xine, SPEED_PAUSE);
          emit modeChange (false, false);
          if (timerId != -1)
            {
              killTimer (timerId);
              timerId = -1;
            }
        }
      else
        {
          xine_set_speed (xine, SPEED_NORMAL);
          emit modeChange (true, false);
          if (timerId == -1)
            timerId = startTimer (500);
        }
    }
}

void
KxineVideo::xineStop ()
{
  xine_stop (xine);

  if (timerId != -1)
    {
      killTimer (timerId);
      timerId = -1;
    }

  emit modeChange (false, false);
  emit positionChange (0);

}

void
KxineVideo::xineRewind ()
{
  // TODO: implement
}

void
KxineVideo::xineFastForward ()
{
  // toggle fast forward speed between 2x and 4x.
  if (xine_get_speed (xine) == SPEED_FAST_2)
    xine_set_speed (xine, SPEED_FAST_4);
  else
    xine_set_speed (xine, SPEED_FAST_2);

  emit modeChange (true, true);
}

void
KxineVideo::xinePrevious ()
{
  if (((xine_get_input_plugin_capabilities(xine)) & INPUT_CAP_CHAPTERS)
         != 0)
    {
      xine_event_t xev;
      xev.type = XINE_EVENT_INPUT_PREVIOUS;
      xine_send_event (xine, &xev);
    }
  else
    {
      // TODO: implement
    }
}

void
KxineVideo::xineNext ()
{
  if (((xine_get_input_plugin_capabilities(xine)) & INPUT_CAP_CHAPTERS)
         != 0)
    {
      xine_event_t xev;
      xev.type = XINE_EVENT_INPUT_NEXT;
      xine_send_event (xine, &xev);
    }
  else 
    {
      QString nextMRL = sidebar -> getNextMRL ();
      if (nextMRL != QString::null)
        xineOpen (nextMRL);
      else
        xineStop ();
    }
}

void
KxineVideo::xineSeek (int pos)
{
  if (xine_is_stream_seekable (xine))
    {
      xine_play (xine, const_cast<char *> (static_cast<const char *> (
                 currentMRL.local8Bit ())), 0, pos);
    }
  emit modeChange (true, false);
}

void
KxineVideo::xineMenu (int id)
{
  xine_event_t xev;
  switch (id)
    {
      case 1:
        xev.type = XINE_EVENT_INPUT_MENU1;
        break;
      case 2:
        xev.type = XINE_EVENT_INPUT_MENU2;
        break;
      case 3:
        xev.type = XINE_EVENT_INPUT_MENU3;
        break;
      default:
        return;
    }
  xine_send_event (xine, &xev);
}

void
KxineVideo::xineSetVolume (int vol)
{
  if (vol < 0)
    vol = 0;
  if (vol > 100)
    vol = 100;

  if(mixer != -1)
    {
      xine_set_audio_property(xine, mixer, vol);
    }
}

void
KxineVideo::xineOpenAutoplayInputPlugin (const QString & aip)
{
  xineStop ();

  application->setOverrideCursor (WaitCursor);

  int num;
  char **mrls =xine_get_autoplay_mrls (xine, const_cast<char *> (
                                      static_cast<const char*> (aip.local8Bit())), &num);
  if (num >= 1)
    {
      xineClose ();
      QStringList s;
      for (int i=0; i<num; i++)
        {
          cerr << "CHAPTER: -->" << mrls[i] << "<--" << endl;
          s.push_back (QString(mrls[i]));
        }
      sidebar -> setChapters (s);
    }
  if (num > 1)
    cerr << "Kxine: warning: got more than one MRL for plugin " << aip << endl;
  
  application->restoreOverrideCursor ();

}

void
KxineVideo::xineExit ()
{
  xine_exit (xine);
}

void
KxineVideo::xineSetProperty (int prop, int value)
{
  videoDriver -> set_property (videoDriver, prop, value);
}

void
KxineVideo::xineSetAudioChannel (int chan)
{
  xine_select_audio_channel (xine, chan);
}

void
KxineVideo::xineSetSubtitleChannel (int chan)
{
  xine_select_spu_channel (xine, chan);
}

void
KxineVideo::xineSetZoom (double zoom)
{
  sizeMultiple = zoom;
}

void
KxineVideo::refresh ()
{
  // fake an expose event
  XExposeEvent eev;
  eev.type = Expose;
  eev.display = xineDisplay;
  eev.window = winId ();
  eev.x = x ();
  eev.y = y ();
  eev.width = width ();
  eev.height = height ();
  eev.count = 0;
  XLockDisplay (xineDisplay);
 videoDriver->gui_data_exchange (videoDriver,
                                 GUI_DATA_EX_EXPOSE_EVENT, &eev);
  XFlush (xineDisplay);
  XUnlockDisplay (xineDisplay);

}

void
KxineVideo::timerEvent (QTimerEvent *tev)
{
  if (tev -> timerId () == timerId)
    {
      emit positionChange (xine_get_current_time (xine));
    }
}

void
KxineVideo::mouseMoveEvent (QMouseEvent *mev)
{
  xine_input_event_t xine_mev;
  xine_mev.event.type = XINE_EVENT_MOUSE_MOVE;
  xine_mev.button = 0;
  int vidx, vidy;
  windowToVideoCoords (mev->x (), mev->y (), &vidx, &vidy);
  xine_mev.x = vidx;
  xine_mev.y = vidy;
  xine_send_event (xine, (xine_event_t *) &xine_mev);
  QWidget::mouseMoveEvent (mev);
}

void
KxineVideo::mousePressEvent (QMouseEvent *mev)
{
  xine_input_event_t xine_mev;
  xine_mev.event.type = XINE_EVENT_MOUSE_BUTTON;
  xine_mev.button = mev->button() == LeftButton ? 1 : 3;
  int vidx, vidy;
  windowToVideoCoords (mev->x (), mev->y (), &vidx, &vidy);
  xine_mev.x = vidx;
  xine_mev.y = vidy;
  xine_send_event (xine, (xine_event_t *) &xine_mev);
  QWidget::mousePressEvent (mev);
}

void
KxineVideo::contextMenuEvent (QContextMenuEvent *cmev)
{
  menu -> getVideoContext () -> popup (cmev -> globalPos ());
  cmev -> accept ();
}

void
KxineVideo::dragEnterEvent (QDragEnterEvent *dev)
{
  dev -> accept(QUriDrag::canDecode (dev));
}

void
KxineVideo::dropEvent (QDropEvent *dev)
{
  QStringList s;
  if (QUriDrag::decodeLocalFiles (dev, s))
    {
      if (s.count () > 0)
      {
        cerr << s.first () << endl;
        sidebar -> clearQueue ();
        sidebar -> enqueueMRLs (s);
        xineNext ();
      }
    }
}

bool
KxineVideo::x11Event (XEvent *)
{
  return false;
}

void
KxineVideo::windowToVideoCoords (int winx, int winy, int *vidx, int *vidy)
{
  x11_rectangle_t rect;

  rect.x = winx;
  rect.y = winy;
  rect.w = 0;
  rect.h = 0;

  if (videoDriver->gui_data_exchange (videoDriver,
                                      GUI_DATA_EX_TRANSLATE_GUI_TO_VIDEO,
                                      (void*)&rect) != -1) {
    /* driver implements gui-to-video coordinate space translation, use it */
    *vidx = rect.x;
    *vidy = rect.y;
    return;
  }

  // FIXME: The fallback code is wrong ;)
  int vidw = videoSize.width();
  int vidh = videoSize.height();
  int winw = width();
  int winh = height();
  *vidx = (winx * vidw) / winw;
  *vidy = (winy * vidh) / winh;
}

