/***************************************************************************
                          kxinevideo.h  -  description
                             -------------------
    begin                : Sun May 19 09:37:48 BST 2002
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KXINEVIDEO_H
#define KXINEVIDEO_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <pthread.h>
#include <qwidget.h>
#include <kapp.h>
#include <xine.h>
#include "kxinemenu.h"
#include "kxinecontrol.h"
#include "kxinesidebar.h"

#include <X11/Xlib.h>

/**
 * KxineVideo
 * This represents the actual connection to xine, and is actually the
 * video window. It is intended as a drop-in component that can be
 * used anywhere, and pretty much all communication is done through
 * signals/slots.
 */ 
class KxineVideo : public QWidget
{
  Q_OBJECT 

  public:
    KxineVideo(KxineSideBar *, KxineMenu *, KxineControl *,
               QWidget* parent=0, const char *name=0);


    inline void setVideoSize (const QSize &newSize)
    { videoSize = newSize; }
    QSize getVideoSize ();

    int  xineGetProperty (int);
    void xineGetPropertyMinMax (int, int*, int*);
    bool xineCheckCapability (int);

    char *xinePeekNextMRL ();
    void resetScreenSaver ();

  public slots:
    virtual void polish ();

    void xineOpen (QString mrl);
    void xineClose (); 
    void xinePlayPause ();
    void xineStop ();
    void xineRewind ();
    void xineFastForward ();
    void xinePrevious ();
    void xineNext ();
    void xineSeek (int);
    void xineMenu (int);
    void xineSetVolume (int);
    void xineOpenAutoplayInputPlugin (const QString&);
    void xineExit ();
    void xineSetProperty (int, int);
    void xineSetAudioChannel (int);
    void xineSetSubtitleChannel (int);
    void xineSetZoom (double);

    void refresh ();

  signals:
    void modeChange (bool playing, bool seeking);
    void fileChange (QString name, int length);
    void positionChange (int pos);

  protected:
    virtual void timerEvent (QTimerEvent *);
    virtual void mouseMoveEvent (QMouseEvent *);
    virtual void mousePressEvent (QMouseEvent *);
    virtual void contextMenuEvent (QContextMenuEvent *);
    virtual void dragEnterEvent (QDragEnterEvent *);
    virtual void dropEvent (QDropEvent *);
    virtual bool x11Event (XEvent *);
    
  /* private: */
  public:
  
    static void destSize_cb (void *v_kxine, int, int, int*, int*);
    static void frameOutput_cb (void *v_kxine, int, int,
                                int*, int*, int*, int*, int*, int*);
    static void xineEvent_cb (void *v_kxine, xine_event_t *);
 
  private:
    void windowToVideoCoords (int, int, int*, int*);

    pthread_t xineXEventThread;
    static void *xineXEventThread_fn (void *v_kxv);

    int shmEventId;
    int timerId;
    int mixer;
    
    xine_t *xine;
    ao_driver_t *audioDriver;
    vo_driver_t *videoDriver;
    config_values_t *xineConf;
    QString currentMRL;
    QSize videoSize;
    double sizeMultiple;

    KxineMenu *menu;
    KxineControl *control;
    KxineSideBar *sidebar;

    Display *xineDisplay;
    

};

#endif
