/***************************************************************************
                          kxinevideocontrol.cpp  -  description
                             -------------------
    begin                : Sun Jul 14 2002
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxinevideocontrol.h"

#include "kxinevideo.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <klocale.h>

#include <iostream>

using namespace std;

KxineVideoControl::KxineVideoControl(QWidget *parent, const char *name ) : QDialog(parent,name)
{
   QGridLayout *grid = new QGridLayout (this, 2, 4, 3, 3);

   c_hue = new QSlider (0, 100, 10, 0, Vertical, this);
   c_sat = new QSlider (0, 100, 10, 0, Vertical, this);
   c_brt = new QSlider (0, 100, 10, 0, Vertical, this);
   c_con = new QSlider (0, 100, 10, 0, Vertical, this);

   QLabel *l_hue = new QLabel (c_hue, i18n ("&Hue"), this);
   QLabel *l_sat = new QLabel (c_sat, i18n ("&Saturation"), this);
   QLabel *l_brt = new QLabel (c_brt, i18n ("&Brightness"), this);
   QLabel *l_con = new QLabel (c_con, i18n ("&Contrast"), this);

   grid -> setColStretch (0, 1);
   grid -> setColStretch (1, 1);
   grid -> setColStretch (2, 1);
   grid -> setColStretch (3, 1);
   
   grid -> addWidget (c_hue, 0, 0, Qt::AlignHCenter);
   grid -> addWidget (c_sat, 0, 1, Qt::AlignHCenter);
   grid -> addWidget (c_brt, 0, 2, Qt::AlignHCenter);
   grid -> addWidget (c_con, 0, 3, Qt::AlignHCenter);
   grid -> addWidget (l_hue, 1, 0, Qt::AlignHCenter);
   grid -> addWidget (l_sat, 1, 1, Qt::AlignHCenter);
   grid -> addWidget (l_brt, 1, 2, Qt::AlignHCenter);
   grid -> addWidget (l_con, 1, 3, Qt::AlignHCenter);

   connect (c_hue, SIGNAL (valueChanged (int)), SLOT (hueMoved (int)));
   connect (c_sat, SIGNAL (valueChanged (int)), SLOT (satMoved (int)));
   connect (c_con, SIGNAL (valueChanged (int)), SLOT (conMoved (int)));
   connect (c_brt, SIGNAL (valueChanged (int)), SLOT (brtMoved (int)));

   setCaption (i18n("Kxine Video Controls"));
   setWFlags (WDestructiveClose);

}

void
KxineVideoControl::enableHue (bool en, int min, int max)
{
  c_hue -> setEnabled (en);
  if (en)
    {
      c_hue -> setMinValue (-max);
      c_hue -> setMaxValue (-min);
    }
}

void
KxineVideoControl::enableContrast (bool en, int min, int max)
{
  c_con -> setEnabled (en);
  if (en)
    {
      c_con -> setMinValue (-max);
      c_con -> setMaxValue (-min);
    }
}

void
KxineVideoControl::enableSaturation (bool en, int min, int max)
{
  c_sat -> setEnabled (en);
  if (en)
    {
      c_sat -> setMinValue (-max);
      c_sat -> setMaxValue (-min);
    }
}

void
KxineVideoControl::enableBrightness (bool en, int min, int max)
{
  c_brt -> setEnabled (en);
  if (en)
    {
      c_brt -> setMinValue (-max);
      c_brt -> setMaxValue (-min);
    }
}

void
KxineVideoControl::setHue (int value)
{
  c_hue -> setValue (-value);
}

void
KxineVideoControl::setContrast (int value)
{
  c_con -> setValue (-value);
}

void
KxineVideoControl::setSaturation (int value)
{
  c_sat -> setValue (-value);
}

void
KxineVideoControl::setBrightness (int value)
{
  c_brt -> setValue (-value);
}

void
KxineVideoControl::hueMoved (int value)
{
   emit propertyChanged (VO_PROP_HUE, -value);
}

void
KxineVideoControl::conMoved (int value)
{
   emit propertyChanged (VO_PROP_CONTRAST, -value);
}

void
KxineVideoControl::satMoved (int value)
{
   emit propertyChanged (VO_PROP_SATURATION, -value);
}

void
KxineVideoControl::brtMoved (int value)
{
   emit propertyChanged (VO_PROP_BRIGHTNESS, -value);
}

void
KxineVideoControl::closeEvent (QCloseEvent *cev)
{
  emit windowClosed ();
  cev -> accept ();
}
